! Programma Monte_Carlo.f90:
! calcola l'integrale della funzione 3D:
! \int_V x*exp[-(x^2+y^2+z^2)]/(x^2+y^2+z^2) dxdydz
! dove V e' un guscio sferico di raggio 1/2.
! 0 <= \theta <= \pi/2
! 0 <= \phi <= \pi/2
! 1/2 <= r <= 1
! L'integrale analitico vale: (\pi/8)*(e^(-1/4)-e^(-1))

program Monte_Carlo

  implicit none
  double precision, parameter :: pi = dacos( -1.d0 )
  double precision, parameter :: R_min = 0.5d0, R_max = 1.d0
  double precision, parameter :: theta_min = 0.d0, theta_max = 0.5d0 * pi 
  double precision, parameter :: phi_min = 0.d0, phi_max = 0.5d0 * pi 
  double precision :: raggio, sum
  double precision :: int_num, int_ex
  double precision :: ran1, ran2, ran3
  integer :: i, idim, np_int, np_tot, Ntot
  integer, allocatable :: arr(:)

  print *,"Numero di punti totali:"
  read *, Ntot

  ! Inizializzazione del generatore di numeri random

  call random_seed(size=idim)
  allocate(arr(idim))
  do i=1,idim
     arr(i)=2*i*i*i+3
  enddo
  call random_seed(put=arr)

  ! Ciclo sui punti

  np_int = 0
  np_tot = 0
  sum = 0.d0
  do while ( np_int < Ntot )
     call random_number( ran1 )
     call random_number( ran2 )
     call random_number( ran3 )
     raggio = dsqrt( ran1 * ran1 + ran2 * ran2 + ran3 * ran3 )
     np_tot = np_tot + 1
     if ( ( R_min <= raggio ) .AND. ( raggio <= R_max ) ) then
        np_int = np_int + 1
        sum = sum + f( ran1, ran2, ran3 )
     endif
  enddo

  int_num = sum / np_tot
  int_ex = ( pi / 8.d0 ) * ( dexp( -0.25d0 ) - dexp( -1.d0 ) )

  print *,"Numero di punti totali: ", np_tot
  print *,"Numero di punti interni al dominio: ", np_int
  print *,"Integrale numerico: ", int_num
  print *,"Integrale esatto: ", int_ex
  print *,"Errore: ", dabs( int_num - int_ex )

contains

  double precision function f( x, y, z )

    double precision :: x, y, z
    double precision :: r2

    r2 = x * x + y * y + z * z
    f = x * dexp( - r2 ) / r2
    return

  end function f

end program Monte_Carlo
