! Programma Trapezi.f90:
! calcola l'integrale della funzione 3D:
! \int_V x*exp[-(x^2+y^2+z^2)]/x^2+y^2+z^2 dxdydz
! dove V e' un guscio sferico di raggio 1/2.
! 0 <= \theta <= \pi/2
! 0 <= \phi <= \pi/2
! 1/2 <= r <= 1

program trapezi

  implicit none
  double precision, parameter :: pi = dacos( -1.d0 )
  double precision, parameter :: R_min = 0.5d0, R_max = 1.d0
  double precision, parameter :: theta_min = 0.d0, theta_max = 0.5d0 * pi 
  double precision, parameter :: phi_min = 0.d0, phi_max = 0.5d0 * pi 
  double precision :: raggio, theta, phi
  double precision :: dr, dtheta, dphi, sum
  double precision :: int_r, int_theta, int_phi, int_num, int_ex
  integer :: Nr, Nphi, Ntheta, ir, itheta, iphi

  print *,"Numero di intervalli lungo r:"
  read *, Nr
  print *,"Numero di intervalli lungo theta:"
  read *, Ntheta
  print *,"Numero di intervalli lungo phi:"
  read *, Nphi

  dr     = ( R_max - R_min ) / Nr
  dtheta = ( theta_max - theta_min ) / Ntheta
  dphi   = ( phi_max - phi_min ) / Nphi

  ! Calcolo integrale lungo r

  sum = 0.5d0 * ( fr( R_min ) + fr( R_max ) )
  do ir = 1, Nr-1
     raggio = R_min + ir * dr
     sum = sum + fr( raggio )
  enddo
  int_r = sum * dr

  ! Calcolo integrale lungo theta

  sum = 0.5d0 * ( ftheta( theta_min ) + ftheta( theta_max ) )
  do itheta = 1, Ntheta-1
     theta = theta_min + itheta * dtheta
     sum = sum + ftheta( theta )
  enddo
  int_theta = sum * dtheta

  ! Calcolo integrale lungo phi

  sum = 0.5d0 * ( fphi( phi_min ) + fphi( phi_max ) )
  do iphi = 1, Nphi-1
     phi = phi_min + iphi * dphi
     sum = sum + fphi( phi )
  enddo
  int_phi = sum * dphi

  int_num = int_r * int_theta * int_phi
  int_ex = ( pi / 8.d0 ) * ( dexp( -0.25d0 ) - dexp( -1.d0 ) )

  print *,"Integrale numerico: ", int_num
  print *,"Integrale esatto: ", int_ex
  print *,"Errore: ", dabs( int_num - int_ex )

contains

  double precision function fr( r )

    double precision :: r

    fr = r * dexp( -( r * r ) )
    return

  end function fr

  double precision function ftheta( theta )

    double precision :: theta

    ftheta = dsin( theta ) * dsin( theta )
    return

  end function ftheta

  double precision function fphi( phi )

    double precision :: phi

    fphi = dcos( phi )
    return

  end function fphi

end program trapezi
